from pygame import event, time, key
from pygame.locals import *
from UserDict import IterableUserDict

class EventHandler:
	"""
	This singleton keeps track of what events are doing what.
	"""
	def __init__(self):
		self.clock = time.Clock()
		self.evs = []
	
	def Pump(self):
		"""
		Pump the events queue.
		"""
		# sleep until we're doing 60 fps
		self.clock.tick(30)
		self.evs = event.get()
	
	def GetEvents(self):
		# pump the events queue
		return self.evs

events = EventHandler()

class EventDict(IterableUserDict):
	"""
	A dictionary for holding held keys and an easy way to check if one is held down.
	"""
	def IsHeld(self, key):
		return (self.has_key(key) and self[key])

class EventMonitor:
	"""
	We subclass this guy to monitor what events are being triggered on a class by class basis.
	"""
	def __init__(self):
		self.quit = False
		self.held = EventDict()
	
	def Pump(self):
		"""
		The update loop fills everthing and runs the routines needed.
		"""
		for e in events.GetEvents():
			if e.type == QUIT:
				self.CallMethod("Quit")
			elif e.type == KEYDOWN:
				self.CallMethod("KeyDown", e)
				self.CallMethod("KeyDown_" + key.name(e.key), e)
				self.held[e.key] = True
			elif e.type == KEYUP:
				self.CallMethod("KeyUp", e)
				self.CallMethod("KeyUp_" + key.name(e.key), e)
				self.held[e.key] = False
			elif e.type == USEREVENT:
				# music
				self.CallMethod("TuneDone", e)
		
		self.CallMethod("KeysHeld", self.held)
		[self.CallMethod("KeyHeld_" + key.name(s)) for s in self.held if self.held[s]]
	
	def CallMethod(self, methodname, args=None):
		"""
		Call a method with the name methodname on self.
		"""
		if hasattr(self, methodname):
			if args != None:
				getattr(self, methodname)(args)
			else:
				getattr(self, methodname)()
	
