from pygame.locals import *
from resources.gfx import echo, echowalk, echowalkup, echowalkdown, echohit, echodie
from Character import Character
from engine.Entity import Entity
from engine import Randomness

class BadRobot(Entity, Character):
	def __init__(self, world, x, y):
		Entity.__init__(self, world)
		
		self.AddAnimation("stand", echo.pufferdata, [self.Normalise])
		self.AddAnimation("walkright", echowalk.pufferdata, [self.Normalise])
		self.AddAnimation("walkleft", echowalk.pufferdata, [self.Normalise, self.Reverse])
		self.AddAnimation("walkup", echowalkup.pufferdata, [self.Normalise])
		self.AddAnimation("walkdown", echowalkdown.pufferdata, [self.Normalise])
		self.AddAnimation("hitl", echohit.pufferdata, [self.Normalise, self.Reverse])
		self.AddAnimation("hitr", echohit.pufferdata, [self.Normalise])
		self.AddAnimation("dead", echodie.pufferdata, [self.Normalise])
		self.Animate("stand")
		
		Randomness.seed()
		
		# could be faster or slightly slower than our player
		self.velocity = 0.01 + Randomness.random() * 0.01 - 0.005
		self.scale = 0.2
		self.position = [x, y]
		self.framerate = 0.6
		
		self.Behaviour = self.Wander
		
		# strength should be a bit more or a bit less than the player's strength
		self.strength = self.world.player.strength + Randomness.randint(0, 6) - 3
		self.plating = self.world.player.plating + Randomness.randint(0, 6) - 3
		
		# randomly color the head and body according to how string and how much plating they have.
		# the less pansy they are, the darker they are
		c = [max(10, 255 - (self.strength + Randomness.randint(0, 30))) for x in range(3)]
		c2 = [max(10, 255 - (self.strength + Randomness.randint(0, 30))) for x in range(3)]
		
		self.ReplaceColor([200, 200, 252], c)
		self.ReplaceColor([201, 200, 255], c2)

		self.Rescale()
	
	def Update(self):
		self.Rescale()
		if self.strength < 0:
			self.Behaviour = self.Dead

		self.Behaviour()
	
	def NearPlayer(self):
		self.Behaviour = self.Attack
	
	def Attack(self):
		"""
		Attack state means we walk towards the player.
		"""
		player = self.world.player
		if player.strength >= 0:
			if self.world.Distance(self.position, player.position) > 0.1:
				self.lastposition = self.position[:]
				
				animated = False
				if player.position[0] < self.position[0] - self.velocity:
					self.Animate("walkleft")
					self.position[0] -= self.velocity
					animated = True
				elif player.position[0] > self.position[0] + self.velocity:
					self.Animate("walkright")
					self.position[0] += self.velocity
					animated = True

				if player.position[1] < self.position[1] - self.velocity:
					if not animated:
						self.Animate("walkup")
					self.position[1] -= self.velocity
				elif player.position[1] > self.position[1] + self.velocity:
					if not animated:
						self.Animate("walkdown")
					self.position[1] += self.velocity
				
		else:
			self.Behaviour = self.Wander
	
	def Wander(self):
		"""
		Random walk all over the place.
		Actually, this does nothing right now.
		"""
	
	def Dead(self):
		"""
		Animate dead and do nothing.
		"""
		self.world.SetDead(self)
		self.Animate("dead")
		

