from engine.Gfx import gfx
from engine.EventHandler import events, EventMonitor
from engine.Entity import Entity
from engine import Randomness
from Scenery import Scenery

class Rock(EventMonitor, Entity, Scenery):
	def __init__(self, world, x, y):
		EventMonitor.__init__(self)
		Entity.__init__(self, world)
		
		Randomness.SetPosition(x, y)
		self.position = [x, y]

		self.scale = 0.1
		
		self.GenerateRock()
		self.AddAnimation("rock", self.data, [self.Normalise])
		self.Animate("rock")
	
	def Update(self):
		self.Rescale()
	
	def GenerateRock(self):
		"""
		Generates a rock using random polygons, dependent on our location.
		Hooray!
		"""
		self.scale = 50 # Randomness.random() * 0.5
		width = Randomness.random()
		height = Randomness.random()
		square = [[-width, height], [-width, 0], [width, 0], [width, height]]
		
		def subdivide(which):
			new = []
			for s in range(len(which) - 1):
				new.append(which[s])
				new.append([(which[s + 1][0] + which[s][0]) / 2.0, (which[s + 1][1] + which[s][1]) / 2.0])
				new.append([which[s + 1][0], which[s + 1][1]])
			return new
		
		def mixup(which):
			for s in range(len(which) - 2):
				which[s + 1][0] = which[s + 1][0] + (Randomness.random() - 0.5) * (width / 2.0)
				which[s + 1][1] = which[s + 1][1] + (Randomness.random() - 0.5) * (height / 2.0)
			return which
		
		iterations = Randomness.randint(1, 4)

		for i in range(iterations):
			square = mixup(square)
			square = subdivide(square)

		square.append([(square[-1][0] + square[0][0]) / 2.0, (square[-1][1] + square[0][1]) / 2.0])
		# square[-1][1] += 0.3
		
		self.data = [[{'outline': [0, 0, 0], 'color': [255, 180, 10], 'width': 1, 'points': square, 'closed': True, 'filled': True}]]
		
		shadow = [square[0]] + square[-iterations * 5:]
		inner = shadow[1:-2]
		inner.reverse()
		for i in inner:
			i[0] -= Randomness.random() * 0.1
			i[1] += Randomness.random() * 0.1
		shadow += inner

		self.data[0].insert(0, {'outline': [0, 0, 0], 'color': [100, 80, 0], 'width': 0, 'points': shadow, 'closed': True, 'filled': True})

		# add the shadow
		# this is a polygon which starts at the -3rd point
